"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var error_1 = require("./utils/error");
var input_validation_1 = require("./utils/input_validation");
var misc_1 = require("./utils/misc");
var MockFirestoreError_1 = require("./utils/MockFirestoreError");
var atob = function (encoded) {
    // Node actually doesn't validate base64 strings.
    // A quick sanity check that is not a fool-proof validation
    if (/[^-A-Za-z0-9+/=]/.test(encoded)) {
        throw new MockFirestoreError_1.MockFirestoreError(error_1.Code.INVALID_ARGUMENT, 'Not a valid Base64 string: ' + encoded);
    }
    return new Buffer(encoded, 'base64').toString('binary');
};
var btoa = function (raw) {
    return new Buffer(raw, 'binary').toString('base64');
};
/** Helper function to assert Uint8Array is available at runtime. */
function assertUint8ArrayAvailable() {
    if (typeof Uint8Array === 'undefined') {
        throw new MockFirestoreError_1.MockFirestoreError(error_1.Code.UNIMPLEMENTED, 'Uint8Arrays are not available in this environment.');
    }
}
/** Helper function to assert Base64 functions are available at runtime. */
function assertBase64Available() { }
/**
 * An immutable object representing an array of bytes.
 */
var MockBlob = /** @class */ (function () {
    function MockBlob(binaryString) {
        assertBase64Available();
        this._binaryString = binaryString;
    }
    /**
     * Creates a new Blob from the given Base64 string, converting it to
     * bytes.
     */
    MockBlob.fromBase64String = function (base64) {
        input_validation_1.validateExactNumberOfArgs('Blob.fromBase64String', arguments, 1);
        input_validation_1.validateArgType('Blob.fromBase64String', 'string', 1, base64);
        assertBase64Available();
        try {
            var binaryString = atob(base64);
            return new MockBlob(binaryString);
        }
        catch (e) {
            throw new MockFirestoreError_1.MockFirestoreError(error_1.Code.INVALID_ARGUMENT, 'Failed to construct Blob from Base64 string: ' + e);
        }
    };
    /**
     * Creates a new Blob from the given Uint8Array.
     */
    MockBlob.fromUint8Array = function (array) {
        input_validation_1.validateExactNumberOfArgs('Blob.fromUint8Array', arguments, 1);
        assertUint8ArrayAvailable();
        if (!(array instanceof Uint8Array)) {
            throw input_validation_1.invalidClassError('MockBlob.fromUint8Array', 'Uint8Array', 1, array);
        }
        // We can't call array.map directly because it expects the return type to
        // be a Uint8Array, whereas we can convert it to a regular array by invoking
        // map on the Array prototype.
        var binaryString = Array.prototype.map
            .call(array, function (char) {
            return String.fromCharCode(char);
        })
            .join('');
        return new MockBlob(binaryString);
    };
    /**
     * Returns the bytes of this Blob as a Base64-encoded string.
     */
    MockBlob.prototype.toBase64 = function () {
        input_validation_1.validateExactNumberOfArgs('Blob.toBase64', arguments, 0);
        assertBase64Available();
        return btoa(this._binaryString);
    };
    /**
     * Returns the bytes of this Blob in a new Uint8Array.
     */
    MockBlob.prototype.toUint8Array = function () {
        input_validation_1.validateExactNumberOfArgs('Blob.toUint8Array', MockBlob.arguments, 0);
        assertUint8ArrayAvailable();
        var buffer = new Uint8Array(this._binaryString.length);
        for (var i = 0; i < this._binaryString.length; i++) {
            buffer[i] = this._binaryString.charCodeAt(i);
        }
        return buffer;
    };
    /**
     * Returns true if this `Blob` is equal to the provided one.
     *
     * @param other The `Blob` to compare against.
     * @return true if this `Blob` is equal to the provided one.
     */
    MockBlob.prototype.isEqual = function (other) {
        return this._binaryString === other._binaryString;
    };
    MockBlob.prototype.toString = function () {
        return 'MockBlob(base64: ' + this.toBase64() + ')';
    };
    /**
     * Actually private to JS consumers of our API, so this function is prefixed
     * with an underscore.
     */
    MockBlob.prototype._compareTo = function (other) {
        return misc_1.primitiveComparator(this._binaryString, other._binaryString);
    };
    return MockBlob;
}());
exports.MockBlob = MockBlob;
//# sourceMappingURL=MockBlob.js.map