import { MockFirebaseFirestore } from '@firebase/app-types';
import { CollectionReference, DocumentChange, DocumentChangeType, DocumentData, DocumentReference, DocumentSnapshot, FieldPath, GetOptions, Query, QuerySnapshot, SnapshotListenOptions, WhereFilterOp, FirestoreDataConverter } from '@firebase/firestore-types';
import { MockCollection } from '.';
import { Mocker } from '../app';
import MockDocumentReference, { ErrorFunction, MockSubscriptionFunction } from './MockDocumentReference';
import { MockQuerySnapshotCallback, MockQuerySnapshotObserver } from './MockQuery';
export interface CollectionMocker<T = DocumentData> extends Mocker {
    docRefs: () => MockDocumentReference<T>[];
    doc(id: string): MockDocumentReference<T>;
    /**
     * Set collection documents
     *
     * @param {MockDocumentReference} doc
     * @memberof CollectionMocker
     */
    setDoc(doc: MockDocumentReference<T>): void;
    /**
     * Delete document and return it's index where it was
     */
    deleteDoc(id: string): number;
    /**
     * Load mock collection content from MockCollection object
     */
    load(collection: MockCollection<T>): void;
    /**
     * Save mock collection data into a new MockCollection object
     *
     * @returns {MockCollection}
     * @memberof CollectionMocker
     */
    save(): MockCollection<T>;
    /**
     * Get a shallow object listing all documents as fields where each field contains
     * the data of the document
     *
     * @returns {{ [documentId: string]: DocumentData}}
     * @memberof CollectionMocker
     */
    getShallowCollection(): {
        [documentId: string]: T | undefined;
    };
    /**
     * Set all documents of the collection by creating them from a single object
     * where eact field is a document and fields value is the data in the document
     * named by the field.
     */
    setShallowCollection(docs: {
        [documentId: string]: T;
    }): void;
    reset(): void;
}
export declare class MockCollectionReference<T = DocumentData> implements CollectionReference<DocumentData> {
    firestore: MockFirebaseFirestore;
    id: string;
    parent: DocumentReference | null;
    /**
     * A string representing the path of the referenced collection (relative
     * to the root of the database).
     */
    get path(): string;
    mocker: CollectionMocker<T>;
    private _docRefs;
    private _snapshotCallbackHandler;
    constructor(firestore: MockFirebaseFirestore, id: string, parent?: DocumentReference | null);
    /**
     * Get a `DocumentReference` for the document within the collection at the
     * specified path. If no path is specified, an automatically-generated
     * unique ID will be used for the returned DocumentReference.
     *
     * @param documentPath A slash-separated path to a document.
     * @return The `DocumentReference` instance.
     */
    doc: (documentPath?: string | undefined) => DocumentReference<T>;
    /**
     * Add a new document to this collection with the specified data, assigning
     * it a document ID automatically.
     *
     * @param data An Object containing the data for the new document.
     * @return A Promise resolved with a `DocumentReference` pointing to the
     * newly created document after it has been written to the backend.
     */
    add: (data: T) => Promise<DocumentReference<T>>;
    /**
     * Returns true if this `CollectionReference` is equal to the provided one.
     *
     * @param other The `CollectionReference` to compare against.
     * @return true if this `CollectionReference` is equal to the provided one.
     */
    isEqual: (other: CollectionReference<DocumentData> | Query<DocumentData>) => boolean;
    /**
     * The `Firestore` for the Firestore database (useful for performing
     * transactions, etc.).
     */
    /**
     * Creates and returns a new Query with the additional filter that documents
     * must contain the specified field and the value should satisfy the
     * relation constraint provided.
     *
     * @param fieldPath The path to compare
     * @param opStr The operation string (e.g "<", "<=", "==", ">", ">=").
     * @param value The value for comparison
     * @return The created Query.
     */
    where: (fieldPath: string | FieldPath, opStr: WhereFilterOp, value: any) => Query<T>;
    /**
     * Creates and returns a new Query that's additionally sorted by the
     * specified field, optionally in descending order instead of ascending.
     *
     * @param fieldPath The field to sort by.
     * @param directionStr Optional direction to sort by ('asc' or 'desc'). If
     * not specified, order will be ascending.
     * @return The created Query.
     */
    orderBy: (fieldPath: string | FieldPath, directionStr?: "desc" | "asc" | undefined) => Query<DocumentData>;
    /**
     * Creates and returns a new Query that's additionally limited to only
     * return up to the specified number of documents.
     *
     * @param limit The maximum number of items to return.
     * @return The created Query.
     */
    limit: (limit: number) => Query<T>;
    limitToLast: (limit: number) => Query<T>;
    /**
     * Creates and returns a new Query that starts at the provided document
     * (inclusive). The starting position is relative to the order of the query.
     * The document must contain all of the fields provided in the orderBy of
     * this query.
     *
     * @param snapshot The snapshot of the document to start at.
     * @param fieldValues The field values to start this query at, in order
     * @return The created Query.
     */
    startAt: (args: any[] | DocumentSnapshot<DocumentData>) => never;
    /**
     * Creates and returns a new Query that starts at the provided fields
     * relative to the order of the query. The order of the field values
     * must match the order of the order by clauses of the query.
     *
     * @param fieldValues The field values to start this query at, in order
     * of the query's order by.
     * @return The created Query.
     */
    /**
     * Creates and returns a new Query that starts after the provided document
     * (exclusive). The starting position is relative to the order of the query.
     * The document must contain all of the fields provided in the orderBy of
     * this query.
     *
     * @param snapshot The snapshot of the document to start after.
     * @return The created Query.
     */
    startAfter: (args: any[] | DocumentSnapshot<T>) => never;
    /**
     * Creates and returns a new Query that starts after the provided fields
     * relative to the order of the query. The order of the field values
     * must match the order of the order by clauses of the query.
     *
     * @param fieldValues The field values to start this query after, in order
     * of the query's order by.
     * @return The created Query.
     */
    /**
     * Creates and returns a new Query that ends before the provided document
     * (exclusive). The end position is relative to the order of the query. The
     * document must contain all of the fields provided in the orderBy of this
     * query.
     *
     * @param snapshot The snapshot of the document to end before.
     * @return The created Query.
     */
    /**
     * Creates and returns a new Query that ends before the provided fields
     * relative to the order of the query. The order of the field values
     * must match the order of the order by clauses of the query.
     *
     * @param fieldValues The field values to end this query before, in order
     * of the query's order by.
     * @return The created Query.
     */
    endBefore: (args: any[] | DocumentSnapshot<DocumentData>) => never;
    /**
     * Creates and returns a new Query that ends at the provided document
     * (inclusive). The end position is relative to the order of the query. The
     * document must contain all of the fields provided in the orderBy of this
     * query.
     *
     * @param snapshot The snapshot of the document to end at.
     * @return The created Query.
     */
    /**
     * Creates and returns a new Query that ends at the provided fields
     * relative to the order of the query. The order of the field values
     * must match the order of the order by clauses of the query.
     *
     * @param fieldValues The field values to end this query at, in order
     * of the query's order by.
     * @return The created Query.
     */
    endAt: (...fieldValues: any[]) => Query<DocumentData>;
    /**
     * Returns true if this `Query` is equal to the provided one.
     *
     * @param other The `Query` to compare against.
     * @return true if this `Query` is equal to the provided one.
     */
    /**
     * Executes the query and returns the results as a QuerySnapshot.
     *
     * Note: By default, get() attempts to provide up-to-date data when possible
     * by waiting for data from the server, but it may return cached data or fail
     * if you are offline and the server cannot be reached. This behavior can be
     * altered via the `GetOptions` parameter.
     *
     * @param options An object to configure the get behavior.
     * @return A Promise that will be resolved with the results of the Query.
     */
    get: (options?: GetOptions | undefined) => Promise<QuerySnapshot<T>>;
    /**
     * Attaches a listener for QuerySnapshot events. You may either pass
     * individual `onNext` and `onError` callbacks or pass a single observer
     * object with `next` and `error` callbacks.
     *
     * NOTE: Although an `onCompletion` callback can be provided, it will
     * never be called because the snapshot stream is never-ending.
     *
     * @param options Options controlling the listen behavior.
     * @param onNext A callback to be called every time a new `QuerySnapshot`
     * is available.
     * @param onError A callback to be called if the listen fails or is
     * cancelled. No further callbacks will occur.
     * @param observer A single object containing `next` and `error` callbacks.
     * @return An unsubscribe function that can be called to cancel
     * the snapshot listener.
     */
    onSnapshot: (optionsOrObserverOrOnNext: SnapshotListenOptions | MockQuerySnapshotObserver | MockQuerySnapshotCallback, observerOrOnNextOrOnError?: MockQuerySnapshotObserver | MockQuerySnapshotCallback | ErrorFunction | undefined, onErrorOrOnCompletion?: ErrorFunction | MockSubscriptionFunction | undefined) => MockSubscriptionFunction;
    fireBatchDocumentChange: (documentChanges: DocumentChange<T>[]) => void;
    withConverter: <U>(converter: FirestoreDataConverter<U>) => CollectionReference<U>;
    mock: () => MockCollectionReference<T>;
    fireSubDocumentChange: (type: DocumentChangeType, snapshot: DocumentSnapshot<T>, oldIndex?: number) => void;
    private getDocs;
    private getQueryDocumentSnapshots;
}
