import { CollectionReference, DocumentChangeType, DocumentData, DocumentReference, DocumentSnapshot, FieldPath, FirestoreError, GetOptions, SetOptions, SnapshotListenOptions, UpdateData, FirestoreDataConverter } from '@firebase/firestore-types';
import { MockFirebaseFirestore } from '@firebase/app-types';
import { MockCollections, MockDocument, Mocker } from '../app';
import { MockCollectionReference } from './MockCollectionReference';
import MockDocumentSnapshot from './MockDocumentSnapshot';
import { MockDocumentChange } from './MockTransaction';
export declare type MockFirestoreFieldPair = [string | FieldPath, any];
export interface SnapshotObserver<T> {
    next?: (snapshot: DocumentSnapshot<T>) => void;
    error?: (error: FirestoreError) => void;
    complete?: () => void;
}
/**
 * Mocker class to change document's state directly without any further consequences. Mocker is used
 * to alter document's internal state for test use case
 */
export interface DocumentMocker<T = DocumentData> extends Mocker {
    collection(id: string): MockCollectionReference<T>;
    setCollection(collection: MockCollectionReference<T>): void;
    load(document: MockDocument<T>): void;
    /**
     * Set document data object
     * @param data A new data for document
     */
    setData(data: T): void;
    getData(): T | undefined;
    saveDocument(): MockDocument<T>;
    saveCollections(): MockCollections<T>;
    listeners(): MockDocumentSnapshotCallback[];
}
export default class MockDocumentReference<T = DocumentData> implements DocumentReference<DocumentData> {
    firestore: MockFirebaseFirestore;
    id: string;
    parent: CollectionReference;
    /**
     * A string representing the path of the referenced document (relative
     * to the root of the database).
     */
    get path(): string;
    get data(): T;
    mocker: DocumentMocker<T>;
    private _data?;
    private _collections;
    private _snapshotCallbackHandler;
    /**
     *
     * @param firestore The `Firestore` for the Firestore database (useful for performing
     * transactions, etc.).
     * @param parent A reference to the Collection to which this DocumentReference belongs.
     */
    constructor(firestore: MockFirebaseFirestore, id: string, parent: CollectionReference);
    /**
     * Gets a `CollectionReference` instance that refers to the collection at
     * the specified path.
     *
     * @param collectionPath A slash-separated path to a collection.
     * @return The `CollectionReference` instance.
     */
    collection: (collectionPath: string) => CollectionReference<T>;
    /**
     * Returns true if this `DocumentReference` is equal to the provided one.
     *
     * @param other The `DocumentReference` to compare against.
     * @return true if this `DocumentReference` is equal to the provided one.
     */
    isEqual: (other: DocumentReference<T>) => boolean;
    /**
     * Writes to the document referred to by this `DocumentReference`. If the
     * document does not yet exist, it will be created. If you pass
     * `SetOptions`, the provided data can be merged into an existing document.
     *
     * @param data A map of the fields and values for the document.
     * @param options An object to configure the set behavior.
     * @return A Promise resolved once the data has been successfully written
     * to the backend (Note that it won't resolve while you're offline).
     */
    set: (data: T, options?: SetOptions | undefined) => Promise<void>;
    setInTransaction: (transactioData: DocumentData, setData: DocumentData, options?: SetOptions | undefined) => DocumentData;
    /**
     * Updates fields in the document referred to by this `DocumentReference`.
     * The update will fail if applied to a document that does not exist.
     *
     * @param data An object containing the fields and values with which to
     * update the document. Fields can contain dots to reference nested fields
     * within the document.
     * @return A Promise resolved once the data has been successfully written
     * to the backend (Note that it won't resolve while you're offline).
     */
    update: (data: string | UpdateData | FieldPath, value?: any, ...moreFieldsAndValues: any[]) => Promise<void>;
    /**
     * Updates fields in the document referred to by this `DocumentReference`.
     * The update will fail if applied to a document that does not exist.
     *
     * @param data An object containing the fields and values with which to
     * update the document. Fields can contain dots to reference nested fields
     * within the document.
     * @return A Promise resolved once the data has been successfully written
     * to the backend (Note that it won't resolve while you're offline).
     */
    updateInTransaction: (transactionData: DocumentData, data: string | UpdateData | FieldPath, value?: any, ...moreFieldsAndValues: any[]) => DocumentData;
    /**
     * Updates fields in the document referred to by this `DocumentReference`.
     * The update will fail if applied to a document that does not exist.
     *
     * Nested fields can be updated by providing dot-separated field path
     * strings or by providing FieldPath objects.
     *
     * @param field The first field to update.
     * @param value The first value.
     * @param moreFieldsAndValues Additional key value pairs.
     * @return A Promise resolved once the data has been successfully written
     * to the backend (Note that it won't resolve while you're offline).
     */
    /**
     * Deletes the document referred to by this `DocumentReference`.
     *
     * @return A Promise resolved once the document has been successfully
     * deleted from the backend (Note that it won't resolve while you're
     * offline).
     */
    delete: () => Promise<void>;
    /**
     * Reads the document referred to by this `DocumentReference`.
     *
     * Note: By default, get() attempts to provide up-to-date data when possible
     * by waiting for data from the server, but it may return cached data or fail
     * if you are offline and the server cannot be reached. This behavior can be
     * altered via the `GetOptions` parameter.
     *
     * @param options An object to configure the get behavior.
     * @return A Promise resolved with a DocumentSnapshot containing the
     * current document contents.
     */
    get: (options?: GetOptions | undefined) => Promise<DocumentSnapshot<T>>;
    /**
     * Attaches a listener for DocumentSnapshot events. You may either pass
     * individual `onNext` and `onError` callbacks or pass a single observer
     * object with `next` and `error` callbacks.
     *
     * NOTE: Although an `onCompletion` callback can be provided, it will
     * never be called because the snapshot stream is never-ending.
     *
     * @param options Options controlling the listen behavior.
     * @param onNext A callback to be called every time a new `DocumentSnapshot`
     * is available.
     * @param onError A callback to be called if the listen fails or is
     * cancelled. No further callbacks will occur.
     * @param observer A single object containing `next` and `error` callbacks.
     * @return An unsubscribe function that can be called to cancel
     * the snapshot listener.
     */
    onSnapshot: <T_1 = DocumentData>(nextObservationOrOptions: SnapshotListenOptions | SnapshotObserver<T_1> | MockDocumentSnapshotCallback, ObserverErrorOrNext?: ErrorFunction | SnapshotObserver<T_1> | MockDocumentSnapshotCallback | undefined, completeOrError?: ErrorFunction | MockSubscriptionFunction | undefined, onComplete?: MockSubscriptionFunction | undefined) => MockSubscriptionFunction;
    withConverter: <U>(converter: FirestoreDataConverter<U>) => DocumentReference<U>;
    fireDocumentChangeEvent: (changeType: DocumentChangeType, oldIndex?: number, cascade?: boolean, callbacks?: MockDocumentSnapshotCallback[] | undefined) => void;
    mock: () => MockDocumentReference<T>;
    commitChange: (type: DocumentChangeType, value: any) => Promise<MockDocumentChange<DocumentData>>;
    /**
     * Private set -method used inside and outside of transaction
     *
     * @private
     * @memberof MockDocumentReference
     */
    private _set;
    private _update;
    private _delete;
}
export declare function extractArguments(data: UpdateData): any[];
export declare type MockSubscriptionFunction = () => void;
export declare type ErrorFunction = (error: Error) => void;
export declare type FirebaseErrorFunction = (error: FirestoreError) => void;
export declare type MockDocumentSnapshotCallback = <T = any>(snapshot: MockDocumentSnapshot<T>) => void;
