"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var utils_1 = require("./utils");
/**
 * A `DocumentSnapshot` contains data read from a document in your Firestore
 * database. The data can be extracted with `.data()` or `.get(<field>)` to
 * get a specific field.
 *
 * For a `DocumentSnapshot` that points to a non-existing document, any data
 * access will return 'undefined'. You can use the `exists` property to
 * explicitly verify a document's existence.
 */
var MockDocumentSnapshot = /** @class */ (function () {
    /**
     *
     * @param ref A `DocumentReference` to the document location.
     * @param id The ID of the document for which this `DocumentSnapshot` contains data.
     */
    function MockDocumentSnapshot(ref, _data) {
        var _this = this;
        this.ref = ref;
        this._data = _data;
        /**
         * Retrieves all fields in the document as an Object. Returns 'undefined' if
         * the document doesn't exist.
         *
         * By default, `FieldValue.serverTimestamp()` values that have not yet been
         * set to their final value will be returned as `null`. You can override
         * this by passing an options object.
         *
         * @param options An options object to configure how data is retrieved from
         * the snapshot (e.g. the desired behavior for server timestamps that have
         * not yet been set to their final value).
         * @return An Object containing all fields in the document or 'undefined' if
         * the document doesn't exist.
         */
        this.data = function (options) {
            return _this._data;
        };
        /**
         * Retrieves the field specified by `fieldPath`. Returns 'undefined' if the
         * document or field doesn't exist.
         *
         * By default, a `FieldValue.serverTimestamp()` that has not yet been set to
         * its final value will be returned as `null`. You can override this by
         * passing an options object.
         *
         * @param fieldPath The path (e.g. 'foo' or 'foo.bar') to a specific field.
         * @param options An options object to configure how the field is retrieved
         * from the snapshot (e.g. the desired behavior for server timestamps that
         * have not yet been set to their final value).
         * @return The data at the specified field location or undefined if no such
         * field exists in the document.
         */
        this.get = function (fieldPath, options) {
            return utils_1.getFieldValueFromData(_this._data, fieldPath);
        };
        /**
         * Returns true if this `DocumentSnapshot` is equal to the provided one.
         *
         * @param other The `DocumentSnapshot` to compare against.
         * @return true if this `DocumentSnapshot` is equal to the provided one.
         */
        this.isEqual = function (other) {
            return _this === other;
        };
    }
    Object.defineProperty(MockDocumentSnapshot.prototype, "exists", {
        /** True if the document exists. */
        get: function () {
            return this._data !== undefined;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MockDocumentSnapshot.prototype, "id", {
        get: function () {
            return this.ref.id;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MockDocumentSnapshot.prototype, "metadata", {
        /**
         * Metadata about this snapshot, concerning its source and if it has local
         * modifications.
         */
        get: function () {
            return {
                hasPendingWrites: false,
                fromCache: false,
                isEqual: function (other) {
                    return other.hasPendingWrites === false && other.fromCache === false;
                },
            };
        },
        enumerable: true,
        configurable: true
    });
    return MockDocumentSnapshot;
}());
exports.default = MockDocumentSnapshot;
//# sourceMappingURL=MockDocumentSnapshot.js.map