"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var MockTimestamp_1 = require("./MockTimestamp");
var NotImplementedYet_1 = require("./utils/NotImplementedYet");
var MockFieldValueType;
(function (MockFieldValueType) {
    MockFieldValueType["ARRAY_UNION"] = "array_union";
    MockFieldValueType["ARRAY_REMOVE"] = "array_remove";
    MockFieldValueType["DELETE"] = "delete";
    MockFieldValueType["INCREMENT"] = "increment";
    MockFieldValueType["TIMESTAMP"] = "timestamp";
})(MockFieldValueType || (MockFieldValueType = {}));
/**
 * Sentinel values that can be used when writing document fields with set()
 * or update().
 */
var MockFieldValue = /** @class */ (function () {
    function MockFieldValue(type) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        this._type = type;
        this._args = args[0];
    }
    Object.defineProperty(MockFieldValue.prototype, "args", {
        get: function () {
            return this._args;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MockFieldValue.prototype, "type", {
        get: function () {
            return this._type;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Returns a sentinel used with set() or update() to include a
     * server-generated timestamp in the written data.
     */
    MockFieldValue.serverTimestamp = function () {
        return MockFieldValue.timestampSentinel;
    };
    /**
     * Returns a sentinel for use with update() to mark a field for deletion.
     */
    MockFieldValue.delete = function () {
        return MockFieldValue.deleteSentinel;
    };
    /**
     * Returns a special value that can be used with set() or update() that tells
     * the server to union the given elements with any array value that already
     * exists on the server. Each specified element that doesn't already exist in
     * the array will be added to the end. If the field being modified is not
     * already an array it will be overwritten with an array containing exactly
     * the specified elements.
     *
     * @param elements The elements to union into the array.
     * @return The FieldValue sentinel for use in a call to set() or update().
     */
    MockFieldValue.arrayUnion = function () {
        var elements = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            elements[_i] = arguments[_i];
        }
        return new MockFieldValue(MockFieldValueType.ARRAY_UNION, elements[0]);
    };
    /**
     * Returns a special value that can be used with set() or update() that tells
     * the server to remove the given elements from any array value that already
     * exists on the server. All instances of each element specified will be
     * removed from the array. If the field being modified is not already an
     * array it will be overwritten with an empty array.
     *
     * @param elements The elements to remove from the array.
     * @return The FieldValue sentinel for use in a call to set() or update().
     */
    MockFieldValue.arrayRemove = function () {
        var elements = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            elements[_i] = arguments[_i];
        }
        return new MockFieldValue(MockFieldValueType.ARRAY_REMOVE, elements[0]);
    };
    /**
     * Returns a special value that can be used with set() or update() that tells
     * the server to increment the field's current value by the given value.
     *
     * If either the operand or the current field value uses floating point
     * precision, all arithmetic will follow IEEE 754 semantics. If both values
     * are integers, values outside of JavaScript's safe number range
     * (`Number.MIN_SAFE_INTEGER` to `Number.MAX_SAFE_INTEGER`) are also subject
     * to precision loss. Furthermore, once processed by the Firestore backend,
     * all integer operations are capped between -2^63 and 2^63-1.
     *
     * If the current field value is not of type 'number', or if the field does
     * not yet exist, the transformation will set the field to the given value.
     *
     * @param n The value to increment by.
     * @return The FieldValue sentinel for use in a call to set() or update().
     */
    MockFieldValue.increment = function (n) {
        return new MockFieldValue(MockFieldValueType.INCREMENT, n);
    };
    /**
     * Returns true if this `FieldValue` is equal to the provided one.
     *
     * @param other The `FieldValue` to compare against.
     * @return true if this `FieldValue` is equal to the provided one.
     */
    MockFieldValue.prototype.isEqual = function (other) {
        if (this._type !== other._type) {
            return false;
        }
        switch (this._type) {
            case MockFieldValueType.DELETE:
            case MockFieldValueType.TIMESTAMP:
                return true;
            case MockFieldValueType.INCREMENT: {
                return other._args[0] === this._args[0];
            }
            case MockFieldValueType.ARRAY_UNION:
            case MockFieldValueType.ARRAY_REMOVE: {
                if (other._args.length !== this._args.length) {
                    return false;
                }
                for (var i = 0; i < this._args.length; i++) {
                    if (other._args[i] !== this._args[i]) {
                        return false;
                    }
                }
                return true;
            }
            default:
                throw new NotImplementedYet_1.NotImplementedYet("MockFieldValue.isEqual have no support for FieldValue type of " + this._type);
        }
    };
    MockFieldValue.deleteSentinel = new MockFieldValue(MockFieldValueType.DELETE);
    MockFieldValue.timestampSentinel = new MockFieldValue(MockFieldValueType.TIMESTAMP);
    return MockFieldValue;
}());
exports.MockFieldValue = MockFieldValue;
/**
 * Preprocess FieldValues from data
 *
 * @export
 * @param {DocumentData} data
 * @returns {DocumentData}
 */
function preprocessData(firestore, data) {
    var result = __assign({}, data);
    for (var key in result) {
        if (result.hasOwnProperty(key)) {
            var value = result[key];
            if (value && value instanceof MockFieldValue) {
                processFieldValue(firestore, data, result, key, value);
            }
            else {
                if (typeof value === 'object') {
                    result[key] = preprocessData(firestore, value);
                }
            }
        }
    }
    return result;
}
exports.preprocessData = preprocessData;
/**
 * Process a single fieldvalue in object
 *
 * @export
 * @param {MockFirebaseFirestoreImpl} firestore
 * @param {DocumentData} sourceData
 * @param {DocumentData} targetData
 * @param {string} key
 * @param {MockFieldValue} fieldValue
 * @returns {DocumentData}
 */
// tslint:disable-next-line
function processFieldValue(firestore, sourceData, targetData, key, fieldValue) {
    switch (fieldValue.type) {
        case MockFieldValueType.DELETE:
            delete targetData[key];
            break;
        case MockFieldValueType.TIMESTAMP:
            var serverTime = firestore.mocker.serverTime;
            targetData[key] = serverTime
                ? typeof serverTime === 'function'
                    ? serverTime()
                    : serverTime
                : MockTimestamp_1.MockTimestamp.now();
            break;
        case MockFieldValueType.ARRAY_UNION:
            {
                var currentValue = targetData[key];
                if (Array.isArray(currentValue)) {
                    var newValues = fieldValue.args;
                    if (Array.isArray(newValues)) {
                        for (var _i = 0, newValues_1 = newValues; _i < newValues_1.length; _i++) {
                            var arg = newValues_1[_i];
                            if (currentValue.indexOf(arg) < 0) {
                                currentValue.push(arg);
                            }
                        }
                    }
                    else {
                        if (currentValue.indexOf(newValues) < 0) {
                            currentValue.push(newValues);
                        }
                    }
                    targetData[key] = currentValue;
                }
                else {
                    // if the field is not an array, all field values will be replaced with the given array values
                    targetData[key] = fieldValue.args;
                }
            }
            break;
        case MockFieldValueType.ARRAY_REMOVE:
            {
                var currentValue = targetData[key];
                if (Array.isArray(currentValue)) {
                    var newValues = fieldValue.args;
                    for (var _a = 0, newValues_2 = newValues; _a < newValues_2.length; _a++) {
                        var arg = newValues_2[_a];
                        var index = currentValue.indexOf(arg);
                        if (index >= 0) {
                            currentValue.splice(index, 1);
                        }
                    }
                    targetData[key] = currentValue;
                }
                else {
                    // if the field is not an array, all field values will be replaced with an empty array
                    targetData[key] = [];
                }
            }
            break;
        case MockFieldValueType.INCREMENT:
            {
                var currentValue = targetData[key];
                if (!currentValue || typeof currentValue !== 'number') {
                    targetData[key] = fieldValue.args[0];
                }
                else {
                    targetData[key] += fieldValue.args[0];
                }
            }
            break;
        default:
            throw new NotImplementedYet_1.NotImplementedYet("processFieldValue " + fieldValue.type);
    }
}
exports.processFieldValue = processFieldValue;
//# sourceMappingURL=MockFieldValue.js.map