import { CollectionReference, FieldPath, FirebaseFirestore, FirestoreError, GetOptions, OrderByDirection, Query, QuerySnapshot, WhereFilterOp, DocumentData, FirestoreDataConverter } from '@firebase/firestore-types';
import MockDocumentReference from './MockDocumentReference';
export interface MockQueryOrderRule {
    fieldPath: string | FieldPath;
    directionStr?: OrderByDirection;
}
export interface MockQueryWhereRule {
    fieldPath: string | FieldPath;
    opStr: WhereFilterOp;
    value: any;
}
export interface MockQueryStartRule {
    fieldValues: any[];
    startAt: 'at' | 'after';
}
export interface MockQueryEndRule {
    fieldValues: any[];
    endAt: 'before' | 'at';
}
interface MockQueryRules {
    /**
     * An array of order rules to define the sorting order of documents. Each rule
     * defines the field for sorting and ascending or descending order to do so. Sorting
     * is read from the left to right.
     *
     * @type {MockQueryOrderRule[]}
     * @memberof MockQueryRules
     */
    order?: MockQueryOrderRule[];
    /**
     * Where rules defines the sub set of the documents that query will fetch. Each
     * where rule will define a field, operation and value to match documents to be
     * be part of query result.
     *
     * @type {MockQueryWhereRule[]}
     * @memberof MockQueryRules
     */
    where?: MockQueryWhereRule[];
    /**
     * Define the set of field values to define the first document ot a document before the
     * the first document to be filtered on result set. Field values define matches for fields
     * defined in order fields.
     *
     * @type {MockQueryStartRule}
     * @memberof MockQueryRules
     */
    start?: MockQueryStartRule;
    /**
     * Define the set of field values to define the last document or document before the last to
     * be filtered on result set. Field values define matches for fields defined in order fields.
     *
     * @type {MockQueryStartRule}
     * @memberof MockQueryRules
     */
    end?: MockQueryEndRule;
    /**
     * Limit the lenght of the result set to be in result set.
     *
     * @type {number}
     * @memberof MockQueryRules
     */
    limit?: number;
}
export declare type MockQuerySnapshotCallback = <T>(snapshot: QuerySnapshot<T>) => void;
/**
 * A `Query` refers to a Query which you can read or listen to. You can also
 * construct refined `Query` objects by adding filters and ordering.
 */
export default class MockQuery<T = DocumentData> implements Query<T> {
    collectionRef: CollectionReference<T>;
    private rules;
    docRefs: MockDocumentReference<T>[];
    /**
     * The `Firestore` for the Firestore database (useful for performing
     * transactions, etc.).
     */
    firestore: FirebaseFirestore;
    private _snapshotCallbackHandler;
    private unsubscribeCollection;
    constructor(collectionRef: CollectionReference<T>, docRefs: MockDocumentReference<T>[], rules?: MockQueryRules);
    reset: () => void;
    /**
     * Creates and returns a new Query with the additional filter that documents
     * must contain the specified field and the value should satisfy the
     * relation constraint provided.
     *
     * @param fieldPath The path to compare
     * @param opStr The operation string (e.g "<", "<=", "==", ">", ">=").
     * @param value The value for comparison
     * @return The created Query.
     */
    where: (fieldPath: string | FieldPath, opStr: WhereFilterOp, value: any) => Query<T>;
    /**
     * Creates and returns a new Query that's additionally sorted by the
     * specified field, optionally in descending order instead of ascending.
     *
     * @param fieldPath The field to sort by.
     * @param directionStr Optional direction to sort by ('asc' or 'desc'). If
     * not specified, order will be ascending.
     * @return The created Query.
     */
    orderBy: (fieldPath: string | FieldPath, directionStr?: OrderByDirection) => Query<T>;
    /**
     * Creates and returns a new Query that's additionally limited to only
     * return up to the specified number of documents.
     *
     * @param limit The maximum number of items to return.
     * @return The created Query.
     */
    limit: (limit: number) => Query<T>;
    limitToLast: (limit: number) => Query<T>;
    /**
     * Creates and returns a new Query that starts at the provided document
     * (inclusive). The starting position is relative to the order of the query.
     * The document must contain all of the fields provided in the orderBy of
     * this query.
     *
     * @param snapshot   The snapshot of the document to start at.
     * or
     * @param fieldValues The field values to start this query at, in order
     * @return The created Query.
     */
    startAt: (...fieldValues: any[]) => Query<T>;
    /**
     * Creates and returns a new Query that starts after the provided document
     * (exclusive). The starting position is relative to the order of the query.
     * The document must contain all of the fields provided in the orderBy of
     * this query.
     *
     * @param snapshot The snapshot of the document to start after.
     * @param fieldValues The field values to start this query after, in order
     * @return The created Query.
     */
    startAfter: (...fieldValues: any[]) => Query<T>;
    /**
     * Creates and returns a new Query that starts after the provided fields
     * relative to the order of the query. The order of the field values
     * must match the order of the order by clauses of the query.
     *
     * @param fieldValues The field values to start this query after, in order
     * of the query's order by.
     * @return The created Query.
     */
    /**
     * Creates and returns a new Query that ends before the provided document
     * (exclusive). The end position is relative to the order of the query. The
     * document must contain all of the fields provided in the orderBy of this
     * query.
     *
     * @param snapshot The snapshot of the document to end before.
     * @param fieldValues The field values to end this query before, in order
     * @return The created Query.
     */
    endBefore: (...fieldValues: any[]) => Query<T>;
    /**
     * Creates and returns a new Query that ends before the provided fields
     * relative to the order of the query. The order of the field values
     * must match the order of the order by clauses of the query.
     *
     * @param fieldValues The field values to end this query before, in order
     * of the query's order by.
     * @return The created Query.
     */
    /**
     * Creates and returns a new Query that ends at the provided document
     * (inclusive). The end position is relative to the order of the query. The
     * document must contain all of the fields provided in the orderBy of this
     * query.
     *
     * @param snapshot The snapshot of the document to end at.
     * @param fieldValues The field values to end this query at, in order
     * @return The created Query.
     */
    endAt: (...fieldValues: any[]) => Query<T>;
    /**
     * Creates and returns a new Query that ends at the provided fields
     * relative to the order of the query. The order of the field values
     * must match the order of the order by clauses of the query.
     *
     * @param fieldValues The field values to end this query at, in order
     * of the query's order by.
     * @return The created Query.
     */
    /**
     * Returns true if this `Query` is equal to the provided one.
     *
     * @param other The `Query` to compare against.
     * @return true if this `Query` is equal to the provided one.
     */
    isEqual: (other: Query<DocumentData>) => boolean;
    /**
     * Executes the query and returns the results as a QuerySnapshot.
     *
     * Note: By default, get() attempts to provide up-to-date data when possible
     * by waiting for data from the server, but it may return cached data or fail
     * if you are offline and the server cannot be reached. This behavior can be
     * altered via the `GetOptions` parameter.
     *
     * @param options An object to configure the get behavior.
     * @return A Promise that will be resolved with the results of the Query.
     */
    get: (options?: GetOptions | undefined) => Promise<QuerySnapshot<T>>;
    /**
     * Attaches a listener for QuerySnapshot events. You may either pass
     * individual `onNext` and `onError` callbacks or pass a single observer
     * object with `next` and `error` callbacks.
     *
     * NOTE: Although an `onCompletion` callback can be provided, it will
     * never be called because the snapshot stream is never-ending.
     *
     * @param options Options controlling the listen behavior.
     * @param onNext A callback to be called every time a new `QuerySnapshot`
     * is available.
     * @param onError A callback to be called if the listen fails or is
     * cancelled. No further callbacks will occur.
     * @param observer A single object containing `next` and `error` callbacks.
     * @return An unsubscribe function that can be called to cancel
     * the snapshot listener.
     */
    onSnapshot: any;
    /**
     * Handle onSnapshot callbacks for document changes that have impact on this query.
     * @memberof MockQuery
     */
    handleCollectionSnapshotChange: (snapshot: QuerySnapshot<T>) => void;
    withConverter: <U>(converter: FirestoreDataConverter<U>) => Query<U>;
    private createClone;
    private getDocumentSnapshots;
    private getFilterDocumentReferences;
    private createStartRule;
    private createEndRule;
}
export interface MockQuerySnapshotObserver {
    next?: (snapshot: QuerySnapshot) => void;
    error?: (error: FirestoreError) => void;
    complete?: () => void;
}
export {};
