"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var MockQueryDocumentSnapshot_1 = require("./MockQueryDocumentSnapshot");
var MockQuerySnapshot_1 = require("./MockQuerySnapshot");
var NotImplementedYet_1 = require("./utils/NotImplementedYet");
var utils_1 = require("./utils");
var CallbackHandler_1 = require("./utils/CallbackHandler");
var matching_1 = require("./utils/matching");
var sortings_1 = require("./utils/sortings");
/**
 * A `Query` refers to a Query which you can read or listen to. You can also
 * construct refined `Query` objects by adding filters and ordering.
 */
var MockQuery = /** @class */ (function () {
    function MockQuery(collectionRef, docRefs, rules) {
        var _this = this;
        if (rules === void 0) { rules = {}; }
        this.collectionRef = collectionRef;
        this.rules = rules;
        this._snapshotCallbackHandler = new CallbackHandler_1.default();
        this.reset = function () {
            _this.unsubscribeCollection();
        };
        /**
         * Creates and returns a new Query with the additional filter that documents
         * must contain the specified field and the value should satisfy the
         * relation constraint provided.
         *
         * @param fieldPath The path to compare
         * @param opStr The operation string (e.g "<", "<=", "==", ">", ">=").
         * @param value The value for comparison
         * @return The created Query.
         */
        this.where = function (fieldPath, opStr, value) {
            var result = _this.createClone();
            var where = result.rules.where || [];
            where.push({
                fieldPath: fieldPath,
                opStr: opStr,
                value: value,
            });
            result.rules.where = where;
            return result;
        };
        /**
         * Creates and returns a new Query that's additionally sorted by the
         * specified field, optionally in descending order instead of ascending.
         *
         * @param fieldPath The field to sort by.
         * @param directionStr Optional direction to sort by ('asc' or 'desc'). If
         * not specified, order will be ascending.
         * @return The created Query.
         */
        this.orderBy = function (fieldPath, directionStr) {
            if (directionStr === void 0) { directionStr = 'asc'; }
            var result = _this.createClone();
            var rules = result.rules.order || [];
            rules.push({
                fieldPath: fieldPath,
                directionStr: directionStr,
            });
            result.rules.order = rules;
            return result;
        };
        /**
         * Creates and returns a new Query that's additionally limited to only
         * return up to the specified number of documents.
         *
         * @param limit The maximum number of items to return.
         * @return The created Query.
         */
        this.limit = function (limit) {
            if (limit <= 0) {
                throw new utils_1.MockFirebaseValidationError('Query limit value must be greater than zero');
            }
            var result = _this.createClone();
            result.rules.limit = limit;
            return result;
        };
        this.limitToLast = function (limit) {
            throw new NotImplementedYet_1.NotImplementedYet("MockQuery.limitToLast");
        };
        /**
         * Creates and returns a new Query that starts at the provided document
         * (inclusive). The starting position is relative to the order of the query.
         * The document must contain all of the fields provided in the orderBy of
         * this query.
         *
         * @param snapshot   The snapshot of the document to start at.
         * or
         * @param fieldValues The field values to start this query at, in order
         * @return The created Query.
         */
        this.startAt = function () {
            var fieldValues = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                fieldValues[_i] = arguments[_i];
            }
            // TODO startAt / startAfter called multiple times?
            _this.rules = _this.createStartRule(fieldValues, 'at');
            return _this;
        };
        // /**
        //  * Creates and returns a new Query that starts at the provided fields
        //  * relative to the order of the query. The order of the field values
        //  * must match the order of the order by clauses of the query.
        //  *
        //  * @param fieldValues The field values to start this query at, in order
        //  * of the query's order by.
        //  * @return The created Query.
        //  */
        // startAt(...fieldValues: any[]): Query;
        /**
         * Creates and returns a new Query that starts after the provided document
         * (exclusive). The starting position is relative to the order of the query.
         * The document must contain all of the fields provided in the orderBy of
         * this query.
         *
         * @param snapshot The snapshot of the document to start after.
         * @param fieldValues The field values to start this query after, in order
         * @return The created Query.
         */
        this.startAfter = function () {
            var fieldValues = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                fieldValues[_i] = arguments[_i];
            }
            _this.rules = _this.createStartRule(fieldValues, 'after');
            return _this;
        };
        /**
         * Creates and returns a new Query that starts after the provided fields
         * relative to the order of the query. The order of the field values
         * must match the order of the order by clauses of the query.
         *
         * @param fieldValues The field values to start this query after, in order
         * of the query's order by.
         * @return The created Query.
         */
        // startAfter(...fieldValues: any[]): Query;
        /**
         * Creates and returns a new Query that ends before the provided document
         * (exclusive). The end position is relative to the order of the query. The
         * document must contain all of the fields provided in the orderBy of this
         * query.
         *
         * @param snapshot The snapshot of the document to end before.
         * @param fieldValues The field values to end this query before, in order
         * @return The created Query.
         */
        this.endBefore = function () {
            var fieldValues = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                fieldValues[_i] = arguments[_i];
            }
            _this.rules = _this.createEndRule(fieldValues, 'before');
            return _this;
        };
        /**
         * Creates and returns a new Query that ends before the provided fields
         * relative to the order of the query. The order of the field values
         * must match the order of the order by clauses of the query.
         *
         * @param fieldValues The field values to end this query before, in order
         * of the query's order by.
         * @return The created Query.
         */
        // endBefore(...fieldValues: any[]): Query;
        /**
         * Creates and returns a new Query that ends at the provided document
         * (inclusive). The end position is relative to the order of the query. The
         * document must contain all of the fields provided in the orderBy of this
         * query.
         *
         * @param snapshot The snapshot of the document to end at.
         * @param fieldValues The field values to end this query at, in order
         * @return The created Query.
         */
        this.endAt = function () {
            var fieldValues = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                fieldValues[_i] = arguments[_i];
            }
            _this.rules = _this.createEndRule(fieldValues, 'at');
            return _this;
        };
        /**
         * Creates and returns a new Query that ends at the provided fields
         * relative to the order of the query. The order of the field values
         * must match the order of the order by clauses of the query.
         *
         * @param fieldValues The field values to end this query at, in order
         * of the query's order by.
         * @return The created Query.
         */
        // endAt(...fieldValues: any[]): Query;
        /**
         * Returns true if this `Query` is equal to the provided one.
         *
         * @param other The `Query` to compare against.
         * @return true if this `Query` is equal to the provided one.
         */
        this.isEqual = function (other) {
            return _this === other;
        };
        /**
         * Executes the query and returns the results as a QuerySnapshot.
         *
         * Note: By default, get() attempts to provide up-to-date data when possible
         * by waiting for data from the server, but it may return cached data or fail
         * if you are offline and the server cannot be reached. This behavior can be
         * altered via the `GetOptions` parameter.
         *
         * @param options An object to configure the get behavior.
         * @return A Promise that will be resolved with the results of the Query.
         */
        this.get = function (options) {
            try {
                var docs = _this.getFilterDocumentReferences();
                return Promise.resolve(new MockQuerySnapshot_1.default(_this, _this.getDocumentSnapshots(docs), []));
            }
            catch (error) {
                return Promise.reject(error);
            }
        };
        /**
         * Attaches a listener for QuerySnapshot events. You may either pass
         * individual `onNext` and `onError` callbacks or pass a single observer
         * object with `next` and `error` callbacks.
         *
         * NOTE: Although an `onCompletion` callback can be provided, it will
         * never be called because the snapshot stream is never-ending.
         *
         * @param options Options controlling the listen behavior.
         * @param onNext A callback to be called every time a new `QuerySnapshot`
         * is available.
         * @param onError A callback to be called if the listen fails or is
         * cancelled. No further callbacks will occur.
         * @param observer A single object containing `next` and `error` callbacks.
         * @return An unsubscribe function that can be called to cancel
         * the snapshot listener.
         */
        this.onSnapshot = (function (optionsOrObserverOrOnNext, observerOrOnNextOrOnError, onErrorOrOnCompletion) {
            if (typeof optionsOrObserverOrOnNext === 'function') {
                _this._snapshotCallbackHandler.add(optionsOrObserverOrOnNext);
                return function () { return _this._snapshotCallbackHandler.remove(optionsOrObserverOrOnNext); };
            }
            throw new NotImplementedYet_1.NotImplementedYet('MockQuery.onSnapshot');
        });
        /**
         * Handle onSnapshot callbacks for document changes that have impact on this query.
         * @memberof MockQuery
         */
        this.handleCollectionSnapshotChange = function (snapshot) {
            var oldDocs = _this.getFilterDocumentReferences();
            _this.docRefs = _this.collectionRef.mocker.docRefs();
            var newDocs = _this.getFilterDocumentReferences();
            var docChanges = [];
            snapshot.docChanges().forEach(function (change) {
                var docId = change.doc.id;
                var oldIndex = oldDocs.findIndex(function (d) { return d.id === docId; });
                var newIndex = newDocs.findIndex(function (d) { return d.id === docId; });
                switch (change.type) {
                    case 'modified':
                    case 'removed': {
                        if (oldIndex >= 0) {
                            docChanges.push(__assign(__assign({}, change), { oldIndex: oldIndex,
                                newIndex: newIndex }));
                        }
                        break;
                    }
                    case 'added': {
                        if (newIndex >= 0) {
                            docChanges.push(__assign(__assign({}, change), { oldIndex: oldIndex,
                                newIndex: newIndex }));
                        }
                        break;
                    }
                    default:
                        throw new Error("Unexpected change type: " + change.type);
                }
            });
            if (docChanges.length > 0) {
                _this._snapshotCallbackHandler.fire(new MockQuerySnapshot_1.default(snapshot.query, _this.getDocumentSnapshots(newDocs), docChanges));
            }
        };
        this.withConverter = function (converter) {
            throw new NotImplementedYet_1.NotImplementedYet('MockQuery.withConverter');
        };
        this.createClone = function () {
            return new MockQuery(_this.collectionRef, _this.docRefs, _this.rules);
        };
        this.getDocumentSnapshots = function (docRefs) {
            if (docRefs === void 0) { docRefs = _this.docRefs; }
            return docRefs.map(function (doc) { return new MockQueryDocumentSnapshot_1.default(doc); });
        };
        this.getFilterDocumentReferences = function () {
            var docs = _this.docRefs;
            var _a = _this.rules, limit = _a.limit, where = _a.where, order = _a.order, start = _a.start, end = _a.end;
            docs = matching_1.filterDocumentsByRules(docs, where);
            docs = sortings_1.sortDocumentsByRules(docs, order);
            if (start) {
                if (!order) {
                    throw new utils_1.MockFirebaseValidationError((start.startAt ? 'StartAt' : 'StartBefore') + " needs to match with query order but order is not defined.");
                }
                var index = utils_1.findIndexForDocument(docs, order.map(function (field) { return field.fieldPath; }), start.fieldValues);
                if (index >= 0) {
                    docs = docs.slice(start.startAt === 'at' ? index : index + 1);
                }
            }
            if (end) {
                if (!order) {
                    throw new utils_1.MockFirebaseValidationError((end.endAt ? 'beforeAt' : 'at') + " needs to match with query order but order is not defined.");
                }
                var index = utils_1.findIndexForDocument(docs, order.map(function (field) { return field.fieldPath; }), end.fieldValues);
                if (index >= 0) {
                    docs = docs.slice(0, end.endAt === 'at' ? index + 1 : index);
                }
            }
            if (limit) {
                docs = docs.slice(0, Math.min(docs.length, limit));
            }
            return docs;
        };
        this.createStartRule = function (fieldValues, startAt) {
            return __assign(__assign({}, _this.rules), { start: {
                    fieldValues: fieldValues,
                    startAt: startAt,
                } });
        };
        this.createEndRule = function (fieldValues, endAt) {
            return __assign(__assign({}, _this.rules), { end: {
                    fieldValues: fieldValues,
                    endAt: endAt,
                } });
        };
        this.firestore = collectionRef.firestore;
        this.docRefs = docRefs.slice();
        this.unsubscribeCollection = this.collectionRef.onSnapshot(this.handleCollectionSnapshotChange);
    }
    return MockQuery;
}());
exports.default = MockQuery;
//# sourceMappingURL=MockQuery.js.map