"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var MockTimestamp_1 = require("./MockTimestamp");
var utils_1 = require("./utils");
/**
 * A `DocumentSnapshot` contains data read from a document in your Firestore
 * database. The data can be extracted with `.data()` or `.get(<field>)` to
 * get a specific field.
 *
 * For a `DocumentSnapshot` that points to a non-existing document, any data
 * access will return 'undefined'. You can use the `exists` property to
 * explicitly verify a document's existence.
 */
var MockQueryDocumentSnapshot = /** @class */ (function () {
    /**
     *
     * @param _query The ID of the document for which this `DocumentSnapshot` contains data.
     */
    function MockQueryDocumentSnapshot(_query) {
        var _this = this;
        this._query = _query;
        /**
         * Retrieves all fields in the document as an Object.
         *
         * By default, `FieldValue.serverTimestamp()` values that have not yet been
         * set to their final value will be returned as `null`. You can override
         * this by passing an options object.
         *
         * @override
         * @param options An options object to configure how data is retrieved from
         * the snapshot (e.g. the desired behavior for server timestamps that have
         * not yet been set to their final value).
         * @return An Object containing all fields in the document.
         */
        this.data = function (options) {
            return _this._query.data;
        };
        /**
         * Retrieves the field specified by `fieldPath`. Returns 'undefined' if the
         * document or field doesn't exist.
         *
         * By default, a `FieldValue.serverTimestamp()` that has not yet been set to
         * its final value will be returned as `null`. You can override this by
         * passing an options object.
         *
         * @param fieldPath The path (e.g. 'foo' or 'foo.bar') to a specific field.
         * @param options An options object to configure how the field is retrieved
         * from the snapshot (e.g. the desired behavior for server timestamps that
         * have not yet been set to their final value).
         * @return The data at the specified field location or undefined if no such
         * field exists in the document.
         */
        this.get = function (fieldPath, options) {
            // TODO options
            return utils_1.getFieldValueFromData(_this._query.data, fieldPath);
        };
        /**
         * Returns true if this `DocumentSnapshot` is equal to the provided one.
         *
         * @param other The `DocumentSnapshot` to compare against.
         * @return true if this `DocumentSnapshot` is equal to the provided one.
         */
        this.isEqual = function (other) {
            return _this === other;
        };
        this.createTime = MockTimestamp_1.MockTimestamp.fromDate(new Date());
        this.updateTime = MockTimestamp_1.MockTimestamp.fromDate(new Date());
        this.readTime = MockTimestamp_1.MockTimestamp.fromDate(new Date());
    }
    Object.defineProperty(MockQueryDocumentSnapshot.prototype, "ref", {
        get: function () {
            return this._query;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MockQueryDocumentSnapshot.prototype, "exists", {
        /** True if the document exists. */
        get: function () {
            return this._query !== undefined;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MockQueryDocumentSnapshot.prototype, "query", {
        get: function () {
            return this._query;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MockQueryDocumentSnapshot.prototype, "id", {
        get: function () {
            return this._query.id;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MockQueryDocumentSnapshot.prototype, "metadata", {
        /**
         * Metadata about this snapshot, concerning its source and if it has local
         * modifications.
         */
        get: function () {
            return {
                hasPendingWrites: false,
                fromCache: false,
                isEqual: function (other) {
                    return other.hasPendingWrites === false && other.fromCache === false;
                },
            };
        },
        enumerable: true,
        configurable: true
    });
    return MockQueryDocumentSnapshot;
}());
exports.default = MockQueryDocumentSnapshot;
//# sourceMappingURL=MockQueryDocumentSnapshot.js.map