"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var MockTimestamp = /** @class */ (function () {
    /**
     * Creates a new timestamp.
     *
     * @param seconds The number of seconds of UTC time since Unix epoch
     *     1970-01-01T00:00:00Z. Must be from 0001-01-01T00:00:00Z to
     *     9999-12-31T23:59:59Z inclusive.
     * @param nanoseconds The non-negative fractions of a second at nanosecond
     *     resolution. Negative second values with fractions must still have
     *     non-negative nanoseconds values that count forward in time. Must be
     *     from 0 to 999,999,999 inclusive.
     */
    function MockTimestamp(seconds, nanoseconds) {
        var _this = this;
        this.valueOf = function () {
            // TODO Check in real case for a match
            return _this._seconds.toString() + "." + _this._nanoseconds.toString();
        };
        this._seconds = seconds;
        this._nanoseconds = nanoseconds;
    }
    /**
     * Creates a new timestamp with the current date, with millisecond precision.
     *
     * @return a new timestamp representing the current date.
     */
    MockTimestamp.now = function () {
        return MockTimestamp.fromDate(new Date());
    };
    /**
     * Creates a new timestamp from the given date.
     *
     * @param date The date to initialize the `Timestamp` from.
     * @return A new `Timestamp` representing the same point in time as the given
     *     date.
     */
    MockTimestamp.fromDate = function (date) {
        return MockTimestamp.fromMillis(date.getTime());
    };
    /**
     * Creates a new timestamp from the given number of milliseconds.
     *
     * @param milliseconds Number of milliseconds since Unix epoch
     *     1970-01-01T00:00:00Z.
     * @return A new `Timestamp` representing the same point in time as the given
     *     number of milliseconds.
     */
    MockTimestamp.fromMillis = function (milliseconds) {
        var seconds = Math.floor(milliseconds / 1000);
        var nanoseconds = (milliseconds - seconds * 1000) * 1000000;
        return new MockTimestamp(seconds, nanoseconds);
    };
    Object.defineProperty(MockTimestamp.prototype, "seconds", {
        get: function () {
            return this._seconds;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MockTimestamp.prototype, "nanoseconds", {
        get: function () {
            return this._nanoseconds;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Returns a new `Date` corresponding to this timestamp. This may lose
     * precision.
     *
     * @return JavaScript `Date` object representing the same point in time as
     *     this `Timestamp`, with millisecond precision.
     */
    MockTimestamp.prototype.toDate = function () {
        return new Date(this.toMillis());
    };
    /**
     * Returns the number of milliseconds since Unix epoch 1970-01-01T00:00:00Z.
     *
     * @return The point in time corresponding to this timestamp, represented as
     *     the number of milliseconds since Unix epoch 1970-01-01T00:00:00Z.
     */
    MockTimestamp.prototype.toMillis = function () {
        return this._seconds * 1000 + this._nanoseconds / 1000000;
    };
    /**
     * Returns true if this `Timestamp` is equal to the provided one.
     *
     * @param other The `Timestamp` to compare against.
     * @return true if this `Timestamp` is equal to the provided one.
     */
    MockTimestamp.prototype.isEqual = function (other) {
        return other.seconds === this._seconds && other.nanoseconds === this._nanoseconds;
    };
    return MockTimestamp;
}());
exports.MockTimestamp = MockTimestamp;
//# sourceMappingURL=MockTimestamp.js.map