import { MockFirebaseApp, MockFirebaseFirestore, MockFirebaseService } from '@firebase/app-types';
import { FirebaseRulesIntepreter } from 'firebase-rules-parser';
import { FirestoreMocker } from './FirestoreMocker';
import { MockCollectionReference } from './MockCollectionReference';
import MockDocumentReference, { MockDocumentSnapshotCallback } from './MockDocumentReference';
import { MockTimestamp } from './MockTimestamp';
import MockTransaction from './MockTransaction';
import { MockWriteBatch } from './MockWritebatch';
import MockQuery from './MockQuery';
import { DocumentData, Settings, PersistenceSettings } from '@firebase/firestore-types';
/**
 * Document object to define database data to be set into a mock
 */
export interface MockDocument<T> {
    data?: T;
    collections?: MockCollections<T>;
    listerners?: MockDocumentSnapshotCallback[];
}
/**
 * A collection of documents in mock database
 */
export interface MockDocuments<T> {
    [documentId: string]: MockDocument<T>;
}
/**
 * Mock Collection object defining documents and listeners that belong to collection
 */
export interface MockCollection<T = any> {
    docs?: MockDocuments<T>;
    listeners?: any[];
}
/**
 * Collection object to define database data to be set into a mock
 */
export interface MockCollections<T = DocumentData> {
    [collectionId: string]: MockCollection<T>;
}
export declare type MockDatabase = MockCollections;
export declare type MockTimestampFunction = () => MockTimestamp;
export interface CollectionObject {
    [documentId: string]: DocumentData;
}
declare type callbackFun = () => void;
/**
 * `Firestore` represents a Firestore Database and is the entry point for all
 * Firestore operations.
 */
export declare class MockFirebaseFirestoreImpl implements MockFirebaseFirestore, MockFirebaseService {
    get rules(): FirebaseRulesIntepreter | undefined;
    /**
     * Mocker object for reading and writing the whole firestore's state without indirect effects. This is a basic tool
     * for setting up the test setup for an unit test.
     */
    mocker: FirestoreMocker;
    /**
     * The `FirebaseApp` associated with this `Firestore` instance.
     */
    app: MockFirebaseApp;
    INTERNAL: any;
    private readonly root;
    private _settings?;
    private _rules?;
    private _rulesHash?;
    private _nextDocumentIdsArray?;
    private _nextDocumentIdsFun?;
    constructor(app: MockFirebaseApp);
    readSettings: () => Settings | undefined;
    /**
     * Specifies custom settings to be used to configure the `Firestore`
     * instance. Must be set before invoking any other methods.
     *
     * @param settings The settings to use.
     */
    settings: (settings: Settings) => void;
    /**
     * Attempts to enable persistent storage, if possible.
     *
     * Must be called before any other methods (other than settings()).
     *
     * If this fails, enablePersistence() will reject the promise it returns.
     * Note that even after this failure, the firestore instance will remain
     * usable, however offline persistence will be disabled.
     *
     * There are several reasons why this can fail, which can be identified by
     * the `code` on the error.
     *
     *   * failed-precondition: The app is already open in another browser tab.
     *   * unimplemented: The browser is incompatible with the offline
     *     persistence implementation.
     *
     * @param settings Optional settings object to configure persistence.
     * @return A promise that represents successfully enabling persistent
     * storage.
     */
    enablePersistence: (settings?: PersistenceSettings | undefined) => Promise<never>;
    /**
     * Gets a `CollectionReference` instance that refers to the collection at
     * the specified path.
     *
     * @param collectionPath A slash-separated path to a collection.
     * @return The `CollectionReference` instance.
     */
    collection: <T = DocumentData>(collectionPath: string) => MockCollectionReference<T>;
    collectionGroup: (collectionId: string) => MockQuery<DocumentData>;
    /**
     * Gets a `DocumentReference` instance that refers to the document at the
     * specified path.
     *
     * @param documentPath A slash-separated path to a document.
     * @return The `DocumentReference` instance.
     */
    doc: <T = DocumentData>(documentPath: string) => MockDocumentReference<T>;
    /**
     * Executes the given updateFunction and then attempts to commit the
     * changes applied within the transaction. If any document read within the
     * transaction has changed, the updateFunction will be retried. If it fails
     * to commit after 5 attempts, the transaction will fail.
     *
     * @param updateFunction The function to execute within the transaction
     * context.
     * @return If the transaction completed successfully or was explicitly
     * aborted (by the updateFunction returning a failed Promise), the Promise
     * returned by the updateFunction will be returned here. Else if the
     * transaction failed, a rejected Promise with the corresponding failure
     * error will be returned.
     */
    runTransaction: <T>(updateFunction: (transaction: MockTransaction) => Promise<T>) => Promise<T>;
    /**
     * Creates a write batch, used for performing multiple writes as a single
     * atomic operation.
     */
    batch: () => MockWriteBatch;
    /**
     * Re-enables use of the network for this Firestore instance after a prior
     * call to disableNetwork().
     *
     * @return A promise that is resolved once the network has been enabled.
     */
    enableNetwork: () => Promise<never>;
    /**
     * Disables network usage for this instance. It can be re-enabled via
     * enableNetwork(). While the network is disabled, any snapshot listeners or
     * get() calls will return results from cache, and any write operations will
     * be queued until the network is restored.
     *
     * @return A promise that is resolved once the network has been disabled.
     */
    disableNetwork: () => Promise<never>;
    private existCall;
    private onGetCall;
    clearPersistence: () => Promise<void>;
    waitForPendingWrites: () => Promise<void>;
    onSnapshotsInSync: (observerOrOnSync: {
        next?: ((value: void) => void) | undefined;
        error?: ((error: Error) => void) | undefined;
        complete?: (() => void) | undefined;
    } | callbackFun) => () => void;
    terminate: () => Promise<void>;
}
export { MockFieldPath } from './MockFieldPath';
export { MockFieldValue } from './MockFieldValue';
export { MockGeoPoint } from './MockGeoPoint';
export { MockTimestamp } from './MockTimestamp';
export { MockBlob } from './MockBlob';
export * from './utils/mocker';
