"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var __1 = require("..");
describe('Document id validation', function () {
    it('Must be valid UTF-8 characters', function () {
        expect(__1.isValidDocumentId('\u0000')).toBeFalsy();
        expect(__1.isValidDocumentId('abc123')).toBeTruthy();
        expect(__1.isValidDocumentId('äöÅ')).toBeTruthy();
    });
    it('Must be no longer than 1,500 bytes', function () {
        var string = new Array(1502).join('a');
        expect(__1.isValidDocumentId(string)).toBeFalsy();
    });
    it('Cannot contain a forward slash (/)', function () {
        expect(__1.isValidDocumentId('a/a')).toBeFalsy();
    });
    it('Cannot solely consist of a single period (.) or double periods (..)', function () {
        expect(__1.isValidDocumentId('.')).toBeFalsy();
        expect(__1.isValidDocumentId('..')).toBeFalsy();
    });
    it('Cannot match the regular expression __.*__', function () {
        expect(__1.isValidDocumentId('__.ab__')).toBeFalsy();
    });
    it('Will allow email addresses', function () {
        expect(__1.isValidDocumentId('matti.meikalainen@gmail.com')).toBeTruthy();
    });
});
describe('Firestore field name validation', function () {
    it('It requised name not be empty', function () {
        expect(__1.isValidFirestoreFieldName('')).toBeFalsy();
    });
    it('The field name contains only the characters a-z, A-Z, 0-9, and underscore (_)', function () {
        expect(__1.isValidFirestoreFieldName('aA0_')).toBeTruthy();
        expect(__1.isValidFirestoreFieldName('ä')).toBeFalsy();
    });
    it('The field name does not start with 0-9', function () {
        expect(__1.isValidFirestoreFieldName('a0')).toBeTruthy();
        expect(__1.isValidFirestoreFieldName('0ä')).toBeFalsy();
    });
    it('Maximum size of a field path is 1,500 bytes', function () {
        expect(__1.isValidFirestoreFieldName('a0')).toBeTruthy();
        var string = new Array(1502).join('a');
        expect(__1.isValidFirestoreFieldName(string)).toBeFalsy();
    });
    it('Will require name in backticks if rules above do not met', function () {
        expect(__1.isValidFirestoreFieldName('`ä`')).toBeTruthy();
        expect(__1.isValidFirestoreFieldName('ä')).toBeFalsy();
    });
});
describe('Firestore field path validation', function () {
    it('Must separate field names with a single period (.)', function () {
        expect(__1.isValidFirestoreFieldPath('a..b')).toBeFalsy();
        expect(__1.isValidFirestoreFieldPath('a.b')).toBeTruthy();
        expect(__1.isValidFirestoreFieldPath('a.b.c')).toBeTruthy();
        expect(__1.isValidFirestoreFieldPath('a.b.c..')).toBeFalsy();
    });
    it('It requised name not be empty', function () {
        expect(__1.isValidFirestoreFieldPath('')).toBeFalsy();
    });
    it('The field name contains only the characters a-z, A-Z, 0-9, and underscore (_)', function () {
        expect(__1.isValidFirestoreFieldPath('aA0_')).toBeTruthy();
        expect(__1.isValidFirestoreFieldPath('ä')).toBeFalsy();
    });
    it('The field name does not start with 0-9', function () {
        expect(__1.isValidFirestoreFieldPath('a0')).toBeTruthy();
        expect(__1.isValidFirestoreFieldPath('0ä')).toBeFalsy();
    });
    it('Maximum size of a field path1,500 bytes', function () {
        expect(__1.isValidFirestoreFieldPath('a0')).toBeTruthy();
        var string = new Array(1502).join('a');
        expect(__1.isValidFirestoreFieldPath(string)).toBeFalsy();
    });
    it('Will require name in backticks if rules above do not met', function () {
        expect(__1.isValidFirestoreFieldPath('`ä`')).toBeTruthy();
        expect(__1.isValidFirestoreFieldPath('`ä')).toBeFalsy();
        expect(__1.isValidFirestoreFieldPath('ä')).toBeFalsy();
    });
});
describe('Firestore document reference path validation', function () {
    it('will have a path with even count of elements', function () {
        expect(__1.isValidDocumentReference('collection')).toBeFalsy();
        expect(__1.isValidDocumentReference('collection/document')).toBeTruthy();
        expect(__1.isValidDocumentReference('collection/doc/subcollection')).toBeFalsy();
        expect(__1.isValidDocumentReference('collection/doc/subcollection/do')).toBeTruthy();
    });
    it('the path is not empty', function () {
        expect(__1.isValidDocumentReference('')).toBeFalsy();
    });
    it('the path will not contain double slashes ', function () {
        expect(__1.isValidDocumentReference('//')).toBeFalsy();
        expect(__1.isValidDocumentReference('collection//doc')).toBeFalsy();
    });
    it('will make sure that path elements are valid', function () {
        expect(__1.isValidDocumentReference('.')).toBeFalsy();
        expect(__1.isValidDocumentReference('..')).toBeFalsy();
        expect(__1.isValidDocumentReference('collection/document\x0000error')).toBeFalsy();
    });
    it('paths max length is set to 1500', function () {
        expect(__1.isValidDocumentReference(new Array(1502).join('a'))).toBeFalsy();
    });
});
describe('Firestore collection refence path validation', function () {
    it('will have a path with odd count of elements', function () {
        expect(__1.isValidCollectionReference('collection')).toBeTruthy();
        expect(__1.isValidCollectionReference('collection/document')).toBeFalsy();
        expect(__1.isValidCollectionReference('collection/doc/subcollection')).toBeTruthy();
        expect(__1.isValidCollectionReference('collection/doc/subcollection/do')).toBeFalsy();
    });
    it('the path is not empty', function () {
        expect(__1.isValidCollectionReference('')).toBeFalsy();
    });
    it('the path will not contain double slashes ', function () {
        expect(__1.isValidCollectionReference('collection//doc')).toBeFalsy();
    });
    it('will make sure that path elements are valid', function () {
        expect(__1.isValidCollectionReference('.')).toBeFalsy();
        expect(__1.isValidCollectionReference('..')).toBeFalsy();
        expect(__1.isValidCollectionReference('\x0000ats/miisu/a')).toBeFalsy();
        expect(__1.isValidCollectionReference('cats/\x00000iisu')).toBeFalsy();
        expect(__1.isValidCollectionReference('cats/miisu/\x00000F/sd')).toBeFalsy();
        expect(__1.isValidCollectionReference('cats/miisu/FF/\x0000d')).toBeFalsy();
    });
    it('paths max length is set to 1500', function () {
        expect(__1.isValidCollectionReference(new Array(1502).join('a'))).toBeFalsy();
    });
});
//# sourceMappingURL=firestoreFieldNameValidation.spec.js.map