"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var __1 = require("..");
var app_1 = require("../../../app");
var MockFieldPath_1 = require("../../MockFieldPath");
var matching_1 = require("../matching");
var firebase = app_1.createFirebaseNamespace();
var firestore = firebase.initializeApp({}).firestore();
describe('Where Clauses', function () {
    describe('simple rules', function () {
        it('will filter documents by exact field match', function () {
            var docs = [
                {
                    data: { a: 3 },
                },
                {
                    data: { a: 2 },
                },
                {
                    data: { a: 1 },
                },
            ];
            var query = matching_1.filterDocumentsByRules(docs, [
                {
                    fieldPath: 'a',
                    opStr: '==',
                    value: 2,
                },
            ]);
            expect(query.length).toBe(1);
        });
        it('will filter documents by exact field-path match', function () {
            var docs = [
                {
                    data: { sub: { a: 3 } },
                },
                {
                    data: { sub: { a: 2 } },
                },
                {
                    data: { sub: { a: 1 } },
                },
            ];
            var query = matching_1.filterDocumentsByRules(docs, [
                {
                    fieldPath: 'sub.a',
                    opStr: '==',
                    value: 2,
                },
            ]);
            expect(query.length).toBe(1);
        });
        it('will filter documents by exact field-path match', function () {
            var docs = [
                {
                    data: { sub: { a: 3 } },
                },
                {
                    data: { sub: { a: 2 } },
                },
                {
                    data: { sub: {} },
                },
            ];
            var query = matching_1.filterDocumentsByRules(docs, [
                {
                    fieldPath: 'sub.a',
                    opStr: '>=',
                    value: 2,
                },
            ]);
            expect(query.length).toBe(2);
        });
    });
    describe('combination rules', function () {
        it('will filter documents by exact field match with two rules', function () {
            var docs = [
                {
                    data: { a: 2, b: 5 },
                },
                {
                    data: {
                        a: 2,
                        b: 3,
                    },
                },
                {
                    data: { a: 1 },
                },
            ];
            var query = matching_1.filterDocumentsByRules(docs, [
                {
                    fieldPath: 'a',
                    opStr: '==',
                    value: 2,
                },
                {
                    fieldPath: 'b',
                    opStr: '>=',
                    value: 3,
                },
            ]);
            expect(query.length).toBe(2);
        });
    });
    describe('Field matching', function () {
        var database = {
            list: {
                docs: {
                    a: {
                        data: {
                            field1: 'cat',
                            sub: {
                                field: 'dog',
                            },
                            field2: 'cow',
                        },
                    },
                },
            },
        };
        it('will find a simple field name - value match', function () {
            firestore.mocker.fromMockDatabase(database);
            var doc = firestore.doc('list/a');
            expect(__1.matchFields(doc, ['field1'], ['cat'])).toBeTruthy();
        });
        it('will return false if the field do not exist', function () {
            firestore.mocker.fromMockDatabase(database);
            var doc = firestore.doc('list/a');
            expect(__1.matchFields(doc, ['field3'], ['cat'])).toBeFalsy();
            expect(__1.matchFields(doc, [new MockFieldPath_1.MockFieldPath('field5', 'daa', 'daa')], ['cat'])).toBeFalsy();
        });
        it('will match values in sub objects pointed with FieldPath', function () {
            firestore.mocker.fromMockDatabase(database);
            var doc = firestore.doc('list/a');
            expect(__1.matchFields(doc, [new MockFieldPath_1.MockFieldPath('sub', 'field')], ['dog'])).toBeTruthy();
        });
        it('will un match values in sub objects pointed with FieldPath that contain wrong value', function () {
            firestore.mocker.fromMockDatabase(database);
            var doc = firestore.doc('list/a');
            expect(__1.matchFields(doc, [new MockFieldPath_1.MockFieldPath('sub', 'field')], ['chicken'])).toBeFalsy();
        });
    });
});
//# sourceMappingURL=matching.spec.js.map