"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var _1 = require(".");
var app_1 = require("../../app");
var MockBlob_1 = require("../MockBlob");
var MockFieldPath_1 = require("../MockFieldPath");
var MockGeoPoint_1 = require("../MockGeoPoint");
var NotImplementedYet_1 = require("./NotImplementedYet");
/**
 * Data types supported by Firestore. Datatypes are defined in an order to match the sorting order
 * of different data types
 */
var MockFirestoreDatatype;
(function (MockFirestoreDatatype) {
    MockFirestoreDatatype[MockFirestoreDatatype["UNDEFINED"] = 0] = "UNDEFINED";
    MockFirestoreDatatype[MockFirestoreDatatype["Null"] = 1] = "Null";
    MockFirestoreDatatype[MockFirestoreDatatype["Boolean"] = 2] = "Boolean";
    MockFirestoreDatatype[MockFirestoreDatatype["Integer"] = 3] = "Integer";
    MockFirestoreDatatype[MockFirestoreDatatype["Float"] = 4] = "Float";
    MockFirestoreDatatype[MockFirestoreDatatype["Numerical"] = 5] = "Numerical";
    MockFirestoreDatatype[MockFirestoreDatatype["Date"] = 6] = "Date";
    MockFirestoreDatatype[MockFirestoreDatatype["Date_Timestamp"] = 7] = "Date_Timestamp";
    MockFirestoreDatatype[MockFirestoreDatatype["Text"] = 8] = "Text";
    MockFirestoreDatatype[MockFirestoreDatatype["Byte"] = 9] = "Byte";
    MockFirestoreDatatype[MockFirestoreDatatype["Reference"] = 10] = "Reference";
    MockFirestoreDatatype[MockFirestoreDatatype["GeoPoint"] = 11] = "GeoPoint";
    MockFirestoreDatatype[MockFirestoreDatatype["Array"] = 12] = "Array";
    MockFirestoreDatatype[MockFirestoreDatatype["Map"] = 13] = "Map";
})(MockFirestoreDatatype = exports.MockFirestoreDatatype || (exports.MockFirestoreDatatype = {}));
/**
 * Generetate sorting function based on given parameters.
 *
 * @param fieldPath Name of a single field or a path object pointing to field to be used for ordering
 * @param directionStr {'asc' | 'desc' | undefined} Sorting direction. Default is `'asc'`.
 */
function querySortFunction(fieldPath, directionStr) {
    if (directionStr === void 0) { directionStr = 'asc'; }
    if (fieldPath === MockFieldPath_1.MockFieldPath.documentId) {
        return function (a, b) {
            // Special case for comparing document ids
            return a.id.localeCompare(b.id);
        };
    }
    return function (a, b) {
        var first = _1.getFieldValue(a, fieldPath);
        var second = _1.getFieldValue(b, fieldPath);
        if (first === undefined || second === undefined) {
            console.warn("Field value not found for path " + fieldPath);
            return 0;
        }
        var firstType = getFirestoreDataType(first);
        var secondType = getFirestoreDataType(second);
        if (firstType !== secondType) {
            return directionStr === 'asc' ? firstType - secondType : -(firstType - secondType);
        }
        // TODO optimize this to resolve a correct compare function on first call
        var result;
        switch (firstType) {
            case MockFirestoreDatatype.Array:
                result = compareArrays(first, second);
                break;
            case MockFirestoreDatatype.Boolean:
                result = first - second;
                break;
            case MockFirestoreDatatype.Float:
            case MockFirestoreDatatype.Integer:
            case MockFirestoreDatatype.Numerical:
                result = first - second;
                break;
            case MockFirestoreDatatype.Text:
                var leftText = cut1500(first);
                var rightText = cut1500(second);
                result = leftText.localeCompare(rightText);
                break;
            case MockFirestoreDatatype.Date:
            case MockFirestoreDatatype.Date_Timestamp: {
                var left = firstType === MockFirestoreDatatype.Date ? app_1.MockTimestamp.fromDate(first) : first;
                var right = secondType === MockFirestoreDatatype.Date ? app_1.MockTimestamp.fromDate(second) : second;
                result = compareTimestamps(left, right);
                break;
            }
            case MockFirestoreDatatype.GeoPoint:
                result = compareGeoPoints(first, second);
                break;
            case MockFirestoreDatatype.Map:
                result = compareObjects(first, second);
                break;
            case MockFirestoreDatatype.Null:
                return 0;
            case MockFirestoreDatatype.Byte:
                result = compareBlobs(first, second);
                break;
            case MockFirestoreDatatype.Reference:
            default:
                throw new NotImplementedYet_1.NotImplementedYet("Query sorting for data type: " + firstType);
        }
        return directionStr === 'asc' ? result : -result;
    };
}
exports.querySortFunction = querySortFunction;
function sortDocumentsByRules(docs, rules) {
    if (!rules) {
        return docs;
    }
    docs.sort(function (a, b) {
        var A = a.data;
        var B = b.data;
        if (A && B) {
            for (var _i = 0, rules_1 = rules; _i < rules_1.length; _i++) {
                var rule = rules_1[_i];
                var fieldPath = rule.fieldPath;
                var value = querySortFunction(fieldPath, rule.directionStr)(a, b);
                if (value !== 0) {
                    return value;
                }
            }
        }
        return 0;
    });
    return docs;
}
exports.sortDocumentsByRules = sortDocumentsByRules;
function cut1500(text) {
    if (text.length < 1500) {
        return text;
    }
    return text.substr(0, 1500);
}
function getFirestoreDataType(value) {
    if (value === undefined) {
        return MockFirestoreDatatype.UNDEFINED;
    }
    if (value === null) {
        return MockFirestoreDatatype.Null;
    }
    if (Array.isArray(value)) {
        return MockFirestoreDatatype.Array;
    }
    switch (typeof value) {
        case 'string':
            return MockFirestoreDatatype.Text;
        case 'number':
            return MockFirestoreDatatype.Numerical;
        case 'boolean':
            return MockFirestoreDatatype.Boolean;
        case 'object': {
            if (value instanceof Date) {
                return MockFirestoreDatatype.Date;
            }
            if (value instanceof app_1.MockTimestamp) {
                return MockFirestoreDatatype.Date_Timestamp;
            }
            if (value instanceof MockGeoPoint_1.MockGeoPoint) {
                return MockFirestoreDatatype.GeoPoint;
            }
            if (value instanceof MockBlob_1.MockBlob) {
                return MockFirestoreDatatype.Byte;
            }
            return MockFirestoreDatatype.Map;
        }
        // TODO bytes, reference, string max 1500 character check
        default:
            throw new NotImplementedYet_1.NotImplementedYet("Query sorting for data type: " + typeof value);
    }
}
exports.getFirestoreDataType = getFirestoreDataType;
/**
 * Compare to objects based on Firestore's sorting rules.
 *
 * @see https://cloud.google.com/firestore/docs/concepts/data-types
 */
function compareObjects(a, b) {
    if (!a || !b || typeof a !== 'object' || typeof b !== 'object') {
        return 0;
    }
    var aKeys = Object.keys(a);
    var bKeys = Object.keys(b);
    var min = Math.min(aKeys.length, bKeys.length);
    for (var i = 0; i < min; i++) {
        if (aKeys[i] < bKeys[i]) {
            return -1;
        }
        if (aKeys[i] > bKeys[i]) {
            return 1;
        }
    }
    var aValues = Object.values(a);
    var bValues = Object.values(b);
    for (var i = 0; i < min; i++) {
        if (aValues[i] < bValues[i]) {
            return -1;
        }
        if (aValues[i] > bValues[i]) {
            return 1;
        }
    }
    if (aKeys.length > bKeys.length) {
        return 1;
    }
    else {
        if (aKeys.length < bKeys.length) {
            return -1;
        }
    }
    return 0;
}
exports.compareObjects = compareObjects;
function compareArrays(a, b) {
    if (!a || !b || !Array.isArray(a) || !Array.isArray(b)) {
        return 0;
    }
    var min = Math.min(a.length, b.length);
    for (var i = 0; i < min; i++) {
        if (a[i] < b[i]) {
            return -1;
        }
        if (a[i] > b[i]) {
            return 1;
        }
    }
    if (a.length > b.length) {
        return 1;
    }
    else {
        if (a.length < b.length) {
            return -1;
        }
    }
    return 0;
}
exports.compareArrays = compareArrays;
function compareTimestamps(a, b) {
    var seconds = a.seconds - b.seconds;
    if (seconds < 0) {
        return -1;
    }
    if (seconds > 0) {
        return 1;
    }
    var nanos = a.nanoseconds - b.nanoseconds;
    if (nanos < 0) {
        return -1;
    }
    if (nanos > 0) {
        return 1;
    }
    return 0;
}
exports.compareTimestamps = compareTimestamps;
function compareGeoPoints(a, b) {
    var latitude = a.latitude - b.latitude;
    if (latitude < 0) {
        return -1;
    }
    if (latitude > 0) {
        return 1;
    }
    var longitude = a.longitude - b.longitude;
    if (longitude < 0) {
        return -1;
    }
    if (longitude > 0) {
        return 1;
    }
    return 0;
}
exports.compareGeoPoints = compareGeoPoints;
function compareBlobs(a, b) {
    return a._compareTo(b);
}
exports.compareBlobs = compareBlobs;
//# sourceMappingURL=sortings.js.map